"""View tests for the ``booking`` app."""
from django.test import TestCase

from django_libs.tests.factories import UserFactory
from django_libs.tests.mixins import ViewTestMixin

from .factories import BookingFactory
from ..models import Booking


class BookingCreateViewTestCase(ViewTestMixin, TestCase):
    def setUp(self):
        self.user = UserFactory()

    def get_view_name(self):
        return 'booking_create'

    def test_view(self):
        self.is_callable()

        data = {
            'gender': 'mr',
            'forename': 'Foo',
            'surname': 'Bar',
            'nationality': 'DE',
            'street1': 'Foostreet 12',
            'city': 'Foocity',
            'zip_code': 'ABC123',
            'country': 'DE',
        }
        self.is_callable(method='post', data=data)
        self.assertEqual(Booking.objects.count(), 1, msg=(
            'One booking should have been created.'))
        self.assertTrue(Booking.objects.all()[0].session.session_key, msg=(
            'Booking should have a session key.'))
        self.is_callable(method='post', data=data)
        self.assertEqual(Booking.objects.count(), 2, msg=(
            'Another booking should have been created.'))

        self.is_callable(method='post', data=data, user=self.user)
        self.assertEqual(Booking.objects.count(), 1, msg=(
            'There should be no bookings left, after the related session has'
            ' been destroyed.'))
        self.assertEqual(self.user.bookings.count(), 1, msg=(
            'User should have a new booking.'))
        self.assertTrue(Booking.objects.all()[0].user.username, msg=(
            'Booking should have a user.'))


class BookingDetailViewTestCase(ViewTestMixin, TestCase):
    def setUp(self):
        self.user = UserFactory()
        self.booking = BookingFactory()

    def get_view_name(self):
        return 'booking_detail'

    def get_view_kwargs(self):
        return {'pk': self.booking.pk}

    def test_view(self):
        self.is_not_callable()
        self.is_not_callable(user=self.user)
        self.booking = BookingFactory(user=self.user)
        self.is_callable(user=self.user)


class BookingListViewTestCase(ViewTestMixin, TestCase):
    def setUp(self):
        self.user = UserFactory()
        BookingFactory(user=self.user)

    def get_view_name(self):
        return 'booking_list'

    def test_view(self):
        self.is_not_callable()
        self.is_callable(user=self.user)



# flake8: noqa
# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Booking.time_period'
        db.add_column(u'booking_booking', 'time_period',
                      self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True),
                      keep_default=False)

        # Adding field 'Booking.time_unit'
        db.add_column(u'booking_booking', 'time_unit',
                      self.gf('django.db.models.fields.CharField')(default='', max_length=64, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Booking.time_period'
        db.delete_column(u'booking_booking', 'time_period')

        # Deleting field 'Booking.time_unit'
        db.delete_column(u'booking_booking', 'time_unit')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'booking.booking': {
            'Meta': {'ordering': "['-creation_date']", 'object_name': 'Booking'},
            'booking_id': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'booking_status': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['booking.BookingStatus']", 'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '2', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_from': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_until': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'forename': ('django.db.models.fields.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '10', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'nationality': ('django.db.models.fields.CharField', [], {'max_length': '2', 'blank': 'True'}),
            'notes': ('django.db.models.fields.TextField', [], {'max_length': '1024', 'blank': 'True'}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['sessions.Session']", 'null': 'True', 'blank': 'True'}),
            'special_request': ('django.db.models.fields.TextField', [], {'max_length': '1024', 'blank': 'True'}),
            'street1': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'}),
            'street2': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'}),
            'surname': ('django.db.models.fields.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'time_period': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'time_unit': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '64', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '10', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'bookings'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'zip_code': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'})
        },
        u'booking.bookingitem': {
            'Meta': {'ordering': "['-booking__creation_date']", 'object_name': 'BookingItem'},
            'booking': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['booking.Booking']"}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'persons': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'quantity': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'})
        },
        u'booking.bookingstatus': {
            'Meta': {'object_name': 'BookingStatus'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'})
        },
        u'booking.bookingstatustranslation': {
            'Meta': {'object_name': 'BookingStatusTranslation'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '16'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'status': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['booking.BookingStatus']"})
        },
        u'booking.extrapersoninfo': {
            'Meta': {'ordering': "['-booking__creation_date']", 'object_name': 'ExtraPersonInfo'},
            'arrival': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'booking': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['booking.Booking']"}),
            'forename': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {'max_length': '1024', 'blank': 'True'}),
            'surname': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'sessions.session': {
            'Meta': {'object_name': 'Session', 'db_table': "'django_session'"},
            'expire_date': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'}),
            'session_data': ('django.db.models.fields.TextField', [], {}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'primary_key': 'True'})
        }
    }

    complete_apps = ['booking']



"""Custom authentication backends for the booking app."""
from django.contrib.auth.backends import ModelBackend

from .models import Booking


class BookingIDBackend(ModelBackend):
    """
    Custom authentication backend that allows login via email and booking ID.

    """
    def authenticate(self, username=None, password=None, **kwargs):
        try:
            booking = Booking.objects.get(
                booking_id=password, user__email=username)
        except Booking.DoesNotExist:
            return None
        return booking.user



"""Tests for the models of the booking app."""
from django.test import TestCase

from . import factories


class BookingTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.booking = factories.BookingFactory()

    def test_instance(self):
        self.assertTrue(self.booking.pk, msg=(
            'Booking model should have been created.'))


class BookingItemTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.booking_item = factories.BookingItemFactory()

    def test_instance(self):
        self.assertTrue(self.booking_item.pk, msg=(
            'Booking item model should have been created.'))


class BookingErrorTestCase(TestCase):
    """Tests for the ``BookingError`` model class."""
    longMessage = True

    def test_instantiation(self):
        bookingerror = factories.BookingErrorFactory()
        self.assertTrue(bookingerror.pk)


class BookingStatusTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.booking_status = factories.BookingStatusFactory()

    def test_instance(self):
        self.assertTrue(self.booking_status.pk, msg=(
            'Booking status model should have been created.'))


class ExtraPersonInfoTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.info = factories.ExtraPersonInfoFactory()

    def test_instance(self):
        self.assertTrue(self.info.pk, msg=(
            'Person info model should have been created.'))



"""Admin classes for the booking app."""
from django.contrib import admin

from simple_translation.admin import TranslationAdmin

from . import models


class BookingAdmin(admin.ModelAdmin):
    list_display = [
        'creation_date', 'booking_status', 'booking_id', 'user', 'email',
        'session', 'date_from', 'date_until',
    ]


class BookingItemAdmin(admin.ModelAdmin):
    list_display = ['booking', 'booked_item', 'quantity', 'persons']


admin.site.register(models.Booking, BookingAdmin)
admin.site.register(models.BookingError)
admin.site.register(models.BookingItem, BookingItemAdmin)
admin.site.register(models.BookingStatus, TranslationAdmin)



"""Utils for the booking app."""
from django.contrib.sessions.models import Session

from .models import Booking


def get_booking(request):
    """
    Returns the booking that is in progress for the current user or None

    We assume that a user can only have one booking that is in-progress.

    TODO: This implementation assumes that there is a status called
    'inprogress' and that there should only be one such booking for a given
    user. We need to see if this can be more generic for future projects.

    :param request: The Request object.

    """
    booking = None
    if request.user.is_authenticated():
        try:
            booking = Booking.objects.get(
                user=request.user,
                booking_status__slug='inprogress')
        except Booking.DoesNotExist:
            # The user does not have any open bookings
            pass
    else:
        session = Session.objects.get(
            session_key=request.session.session_key)
        try:
            booking = Booking.objects.get(session=session)
        except Booking.DoesNotExist:
            # The user does not have any bookings in his session
            pass
    return booking


def persist_booking(booking, user):
    """
    Ties an in-progress booking from a session to a user when the user logs in.

    If we don't do this, the booking will be lost, because on a login, the
    old session will be deleted and a new one will be created. Since the
    booking has a FK to the session, it would be deleted as well when the user
    logs in.

    We assume that a user can only have one booking that is in-progress.
    Therefore we will delete any existing in-progress bookings of this user
    before tying the one from the session to the user.

    TODO: Find a more generic solution for this, as this assumes that there is
    a status called inprogress and that a user can only have one such booking.

    :param booking: The booking that should be tied to the user.
    :user: The user the booking should be tied to.

    """
    if booking is not None:
        existing_bookings = Booking.objects.filter(
            user=user, booking_status__slug='inprogress').exclude(
            pk=booking.pk)
        existing_bookings.delete()

        booking.session = None
        booking.user = user
        booking.save()



"""Form tests for the ``booking`` app."""
from django.test import TestCase

from .factories import BookingStatusFactory
from ..forms import BookingForm
from ..models import Booking


class BookingFormTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.status = BookingStatusFactory()

    def test_form(self):
        form = BookingForm()
        self.assertTrue(form, msg=('Form has been initiated.'))

        data = {
            'gender': 'mr',
            'forename': 'Foo',
            'nationality': 'DE',
            'street1': 'Foostreet 12',
            'city': 'Foocity',
            'zip_code': 'ABC123',
            'country': 'DE',
        }
        form = BookingForm(data=data)
        self.assertFalse(form.is_valid(), msg=('Form should be invalid.'))

        data.update({'surname': 'Bar'})
        form = BookingForm(data=data)
        self.assertTrue(form.is_valid(), msg=('Form should be valid.'))

        form.save()
        self.assertEqual(Booking.objects.count(), 1, msg=(
            'One booking should have been created.'))
        self.assertEqual(
            Booking.objects.all()[0].booking_status.slug,
            'pending', msg=('Slug of status should be pending.'))



"""Registering translated models for the ``booking`` app."""
from simple_translation.translation_pool import translation_pool

from .models import BookingStatus, BookingStatusTranslation


translation_pool.register_translation(BookingStatus, BookingStatusTranslation)



"""Models for the ``booking`` app."""
from django.conf import settings
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _

from django_libs.models_mixins import SimpleTranslationMixin
from international.models import countries


class BookingStatus(SimpleTranslationMixin, models.Model):
    """
    Master data containing all booking status.
    For translatable fields check ``BookingStatusTranslation``.

    :slug: A unique slug identifier.

    """
    slug = models.SlugField(
        verbose_name=_('Slug'),
    )

    def __unicode__(self):
        return self.slug


class BookingStatusTranslation(models.Model):
    """
    Translatable fields of the ``BookingStatus`` model.

    :name: The name of the status.

    Needed by simple translation:
    :language: The language of the translation.
    :status: The status this translation belongs to.

    """
    name = models.CharField(
        max_length=128,
        verbose_name=_('Name'),
    )

    # needed by simple translation
    language = models.CharField(max_length=16)
    status = models.ForeignKey(BookingStatus)

    def __unicode__(self):
        return self.name


class Booking(models.Model):
    """
    Model to contain information about a booking.

    Note, that on the model itself, most of the attributes are blank=True.
    We need this behaviour to be able to create empty temporary bookings.
    You will have to take care of the field being required or not in a
    ModelForm yourself.

    :user (optional): Connection to Django's User model.
    :session (optional): Stored session to identify anonymous users.
    :gender (optional): Gender of the user.
    :title (optional): Title of the user.
    :forename (optional): First name of the user.
    :surname (optional): Last name of the user.
    :nationality (optional): The nationality of the user.
    :street1 (optional): Street address of the user.
    :street2 (optional): Additional street address of the user.
    :city (optional): City of the user's address.
    :zip_code (optional): ZIP of the user's address.
    :country (optional): Country of the user's address.
    :phone (optional): Phone number of the user.
    :email: Email of the user.
    :special_request (optional): A special request of the customer.
    :date_from (optional): From when the booking is active.
    :date_until (optional): Until when the booking is active.
    :time_period (optional): How long the period from date_from will be.
      e.g.: 10 (days).
    :time_unit (optional): What unit of time the period is of. e.g. nights or
      days.
    :creation_date: Date of the booking.
    :booking_id (optional): Custom unique booking identifier.
    :booking_status: Current status of the booking.
    :notes (optional): Staff notes.
    :total (optional): Field for storing a total of all items.
    :currency (optional): If total is uses, we usually also need a currency.

    """
    user = models.ForeignKey(
        'auth.User',
        verbose_name=_('User'),
        related_name='bookings',
        blank=True, null=True,
    )

    session = models.ForeignKey(
        'sessions.Session',
        verbose_name=_('Session'),
        blank=True, null=True,
    )

    gender = models.CharField(
        max_length=10,
        verbose_name=_('Gender'),
        choices=(
            ('mrs', _('Mrs')),
            ('mr', _('Mr')),
        ),
        blank=True,
    )

    title = models.CharField(
        max_length=10,
        verbose_name=_('Title'),
        choices=(
            ('dr', _('Dr.')),
            ('prof', _('Prof.')),
        ),
        blank=True,
    )

    forename = models.CharField(
        verbose_name=_('First name'),
        max_length=20,
        blank=True,
    )

    surname = models.CharField(
        verbose_name=_('Last name'),
        max_length=20,
        blank=True,
    )

    nationality = models.CharField(
        max_length=2,
        verbose_name=_('Nationality'),
        choices=countries,
        blank=True,
    )

    street1 = models.CharField(
        verbose_name=_('Street 1'),
        max_length=256,
        blank=True,
    )

    street2 = models.CharField(
        verbose_name=_('Street 2'),
        max_length=256,
        blank=True,
    )

    city = models.CharField(
        verbose_name=_('City'),
        max_length=256,
        blank=True,
    )

    zip_code = models.CharField(
        verbose_name=_('ZIP/Postal code'),
        max_length=256,
        blank=True,
    )

    country = models.CharField(
        max_length=2,
        verbose_name=_('Country'),
        choices=countries,
        blank=True,
    )

    email = models.EmailField(
        verbose_name=_('Email'),
        blank=True,
    )

    phone = models.CharField(
        verbose_name=_('Phone'),
        max_length=256,
        blank=True,
    )

    special_request = models.TextField(
        max_length=1024,
        verbose_name=_('Special request'),
        blank=True,
    )

    date_from = models.DateTimeField(
        verbose_name=_('From'),
        blank=True, null=True,
    )

    date_until = models.DateTimeField(
        verbose_name=_('Until'),
        blank=True, null=True,
    )

    creation_date = models.DateTimeField(
        verbose_name=_('Creation date'),
        auto_now_add=True,
    )

    booking_id = models.CharField(
        max_length=100,
        verbose_name=_('Booking ID'),
        blank=True,
    )

    booking_status = models.ForeignKey(
        'booking.BookingStatus',
        verbose_name=('Booking status'),
        blank=True, null=True,
    )

    notes = models.TextField(
        max_length=1024,
        verbose_name=('Notes'),
        blank=True,
    )

    time_period = models.PositiveIntegerField(
        verbose_name=_('Time period'),
        blank=True, null=True,
    )

    time_unit = models.CharField(
        verbose_name=_('Time unit'),
        default=getattr(settings, 'BOOKING_TIME_INTERVAL', ''),
        max_length=64,
        blank=True,
    )

    total = models.DecimalField(
        max_digits=36,
        decimal_places=28,
        verbose_name=_('Total'),
        blank=True, null=True,
    )

    currency = models.CharField(
        verbose_name=_('Currency'),
        max_length=128,
        blank=True,
    )

    class Meta:
        ordering = ['-creation_date']

    def __unicode__(self):
        return '#{} ({})'.format(self.booking_id or self.pk,
                                 self.creation_date)


class BookingError(models.Model):
    """
    Holds information about an error during a booking process.

    This can be particularly useful, when many of the processes are automated
    or reliant on a third party app or API. You then can store the returned
    values directly into this model and have easy access and reference to the
    actual booking.

    :booking: The booking during this error occurred.
    :message: The short error message, that you need to store.
    :details: A more in depth text about the error or any kind of additional
      information, e.g. a traceback.
    :date: The time and date this error occured.

    """
    booking = models.ForeignKey(
        Booking,
        verbose_name=_('Booking'),
    )
    message = models.CharField(
        verbose_name=_('Message'),
        max_length=1000,
        blank=True,
    )
    details = models.TextField(
        verbose_name=_('Details'),
        max_length=4000,
        blank=True,
    )

    date = models.DateTimeField(
        verbose_name=_('Date'),
        auto_now_add=True,
    )

    def __unicode__(self):
        return '[{0}] {1} - {2}'.format(self.date, self.booking.booking_id,
                                        self.message)


class BookingItem(models.Model):
    """
    Model to connect a booking with a related object.

    :quantity: Quantity of booked items.
    :persons (optional): Quantity of persons, who are involved in this booking.
    :subtotal (optional): Field for storing subtotals for each individual item.
    :booked_item: Connection to related booked item.
    :booking: Connection to related booking.

    """
    quantity = models.PositiveIntegerField(
        default=1,
        verbose_name=_('Quantity'),
    )

    persons = models.PositiveIntegerField(
        verbose_name=_('Persons'),
        blank=True, null=True,
    )

    subtotal = models.DecimalField(
        max_digits=36,
        decimal_places=28,
        verbose_name=_('Subtotal'),
        blank=True, null=True,
    )

    # GFK 'booked_item'
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    booked_item = generic.GenericForeignKey('content_type', 'object_id')

    booking = models.ForeignKey(
        'booking.Booking',
        verbose_name=_('Booking'),
    )

    class Meta:
        ordering = ['-booking__creation_date']

    def __unicode__(self):
        return '{} ({})'.format(self.booking, self.booked_item)


class ExtraPersonInfo(models.Model):
    """
    Model to add extra information of persons/guests to a booking.

    :forename: First name of the user.
    :surname: Last name of the user.
    :arrival: Arrival date of the guest.
    :booking: Connection to related booking.
    :message: An additional message regarding this person.

    """
    forename = models.CharField(
        verbose_name=_('First name'),
        max_length=20,
    )

    surname = models.CharField(
        verbose_name=_('Last name'),
        max_length=20,
    )

    arrival = models.DateTimeField(
        verbose_name=_('Arrival'),
        blank=True, null=True,
    )

    booking = models.ForeignKey(
        'booking.Booking',
        verbose_name=_('Booking'),
    )

    message = models.TextField(
        max_length=1024,
        verbose_name=_('Message'),
        blank=True,
    )

    class Meta:
        ordering = ['-booking__creation_date']

    def __unicode__(self):
        return '{} {} ({})'.format(self.forename, self.surname, self.booking)
